@@ -1,4 +1,4 @@
-/* $OpenBSD: ssl_sigalgs.c,v 1.20 2019/04/01 02:09:21 beck Exp $ */
+/* $OpenBSD: ssl_sigalgs.c,v 1.20.8.1 2020/08/10 18:59:47 tb Exp $ */
 /*
  * Copyright (c) 2018-2019 Bob Beck <beck@openbsd.org>
  *
@@ -322,6 +322,12 @@ ssl_sigalg_select(SSL *s, EVP_PKEY *pkey)
 		    tls_sigalgs_len)) == NULL)
 			continue;
 
+		/* RSA cannot be used without PSS in TLSv1.3. */
+		if (TLS1_get_version(s) >= TLS1_3_VERSION &&
+		    sigalg->key_type == EVP_PKEY_RSA &&
+		    (sigalg->flags & SIGALG_FLAG_RSA_PSS) == 0)
+			continue;
+
 		if (ssl_sigalg_pkey_ok(sigalg, pkey, check_curve))
 			return sigalg;
 	}
